from typing import Union

import requests
from requests_html import HTMLSession, HTMLResponse, HTML

from scylla.loggings import logger


class Worker:

    def __init__(self):
        """Initialize the worker object

        """

        self.session = HTMLSession()

    def get_html(self, url: str, render_js: bool = True) -> Union[HTML, None]:
        """Get html from a specific URL

        :param url: the URL
        :param render_js: [whether to render js], defaults to True
        :param render_js: bool, optional
        :return: [the HTML string]
        :rtype: str
        """

        try:
            # TODO: load config for timeout
            response: HTMLResponse = self.session.get(url, timeout=30)
        except requests.RequestException:
            logger.warning('[Worker] Cannot get this url: ' + url)
            return None

        if response.ok:
            if render_js:
                print('starting render js...')
                response.html.render(wait=1.5, timeout=10.0)
                print('end render js...')
            return response.html
        else:
            return None



import re

from requests_html import HTML

from scylla.database import ProxyIP
from .base_provider import BaseProvider


class HttpProxyProvider(BaseProvider):

    def parse(self, html: HTML) -> [ProxyIP]:
        ip_list: [ProxyIP] = []

        for ip_row in html.find('table.proxytbl tr'):

            ip_element = ip_row.find('td:nth-child(1)', first=True)
            port_element = ip_row.find('td:nth-child(2)', first=True)

            try:
                if ip_element and port_element:
                    port_str = re.search(r'//]]> (\d+)', port_element.text).group(1)

                    p = ProxyIP(ip=ip_element.text, port=port_str)

                    ip_list.append(p)
            except AttributeError:
                pass

        return ip_list

    def urls(self) -> [str]:
        return [
            'https://proxyhttp.net/free-list/proxy-anonymous-hide-ip-address/',
            'https://proxyhttp.net/',
            'https://proxyhttp.net/free-list/anonymous-server-hide-ip-address/2#proxylist',
        ]

    @staticmethod
    def should_render_js() -> bool:
        return True



import datetime

from peewee import Model, CharField, DateTimeField, BooleanField, FloatField, IntegerField, SqliteDatabase

from scylla.loggings import logger

_db = None


def create_connection() -> SqliteDatabase:
    """
    create a database connection
    :rtype: SqliteDatabase
    """
    global _db
    if _db:
        return _db
    else:
        logger.debug('create new connection')
        _db = SqliteDatabase('scylla.db')
        return _db


def create_db_tables():
    db = create_connection()
    db.create_tables([ProxyIP])


class BaseModel(Model):
    class Meta:
        database = create_connection()


class ProxyIP(BaseModel):
    class Meta:
        table_name = 'proxy_ips'

    ip = CharField(unique=True)
    port = IntegerField()
    is_valid = BooleanField(default=False)
    created_at = DateTimeField(default=datetime.datetime.now)
    updated_at = DateTimeField(default=datetime.datetime.now)
    latency = FloatField()
    stability = FloatField()
    is_anonymous = BooleanField(default=False)
    location = CharField(null=True)
    organization = CharField(null=True)
    region = CharField(null=True)
    country = CharField(null=True)
    city = CharField(null=True)

    def __str__(self):
        return '[database.ProxyIP ip: {}, port: {}, is_valid: {}, latency: {}]' \
            .format(self.ip, self.port, self.is_valid, self.latency)

    def __repr__(self):
        return self.__str__()



import re

from requests_html import HTML

from scylla.database import ProxyIP
from .base_provider import BaseProvider


class A2uProvider(BaseProvider):

    def urls(self) -> [str]:
        return [
            'https://raw.githubusercontent.com/a2u/free-proxy-list/master/free-proxy-list.txt',
        ]

    def parse(self, html: HTML) -> [ProxyIP]:
        ip_list: [ProxyIP] = []

        text = html.raw_html

        ip_port_str_list = re.findall(r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}:\d{2,5}', text.decode('utf-8'))

        for ip_port in ip_port_str_list:

            ip = re.search(r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}', ip_port).group(0)
            port = re.search(r':(\d{2,5})', ip_port).group(1)

            if ip and port:
                p = ProxyIP(ip=ip, port=port)
                ip_list.append(p)

        return ip_list

    @staticmethod
    def should_render_js() -> bool:
        return False



import logging
import sys

_formatter = logging.Formatter(fmt="%(asctime)s %(levelname)s: %(message)s",
                               datefmt="%Y-%m-%d - %H:%M:%S")
_ch = logging.StreamHandler(sys.stdout)
_ch.setLevel(logging.DEBUG)
_ch.setFormatter(_formatter)

# _fh = logging.FileHandler("mylog.log", "w")
# _fh.setLevel(logging.DEBUG)
# _fh.setFormatter(_formatter)

logger = logging.getLogger('scylla')
logger.setLevel(logging.DEBUG)

logger.addHandler(_ch)
# logger.addHandler(_fh)



from .a2u_provider import A2uProvider
from .base_provider import BaseProvider
from .cool_proxy_provider import CoolProxyProvider
from .data5u_provider import Data5uProvider
from .free_proxy_list_provider import FreeProxyListProvider
from .http_proxy_provider import HttpProxyProvider
from .kuaidaili_provider import KuaidailiProvider
from .spys_me_provider import SpyMeProvider
from .spys_one_provider import SpysOneProvider
from .xici_provider import XiciProvider

all_providers = [
    A2uProvider,
    CoolProxyProvider,
    Data5uProvider,
    FreeProxyListProvider,
    HttpProxyProvider,
    KuaidailiProvider,
    SpyMeProvider,
    SpysOneProvider,
    XiciProvider
]

# Provider references:
# https://github.com/franklingu/proxypool/issues/2



import datetime

from scylla.database import ProxyIP
from scylla.validator import Validator
from .loggings import logger


def save_ip(p: ProxyIP):
    basic_query = ProxyIP.select().where(ProxyIP.ip == p.ip)
    count = basic_query.count()
    if count == 0:
        logger.debug('Creating new ip record: ' + p.__str__())
        p.save()
    else:
        logger.debug('Update an existing ip record: ' + p.__str__())

        ProxyIP.update(latency=p.latency, stability=p.stability, is_valid=p.is_valid,
                       is_anonymous=p.is_anonymous, updated_at=datetime.datetime.now()).where(
            ProxyIP.ip == p.ip).execute()

        logger.debug('Saved: ' + p.__str__())


def validate_proxy_ip(p: ProxyIP):
    logger.debug('Validating ip: {}'.format(p.ip))
    v = Validator(host=p.ip, port=int(p.port))

    try:
        v.validate()
    except KeyboardInterrupt:
        logger.info('KeyboardInterrupt terminates validate_proxy_ip: ' + p.ip)

    meta = v.meta if v.meta else {}
    validated_ip = ProxyIP(ip=p.ip, port=p.port, **meta)
    # save valid ip into database
    validated_ip.latency = v.latency
    validated_ip.stability = v.success_rate
    validated_ip.is_valid = v.valid
    validated_ip.is_anonymous = v.anonymous

    logger.debug('Save valid ip into database: \n' + validated_ip.__str__())

    save_ip(validated_ip)

    logger.debug('Finish validating ip: {}'.format(validated_ip.ip))



from typing import Union

_config_data = {}


def _config_data_instance():
    global _config_data
    return _config_data


def set_config(key: str, value: str):
    _config_data_instance()[key] = value


def get_config(key: str, default: str = None) -> Union[str, None]:
    try:
        return _config_data_instance()[key]
    except KeyError:
        return default


def batch_set_config(**kwargs):
    for k, v in kwargs.items():
        set_config(k, v)



from requests_html import HTML

from scylla.database import ProxyIP
from scylla.providers import BaseProvider


class KuaidailiProvider(BaseProvider):

    def urls(self) -> [str]:
        return [
            'https://www.kuaidaili.com/free/inha/1/',
        ]

    def parse(self, html: HTML) -> [ProxyIP]:
        ip_list: [ProxyIP] = []

        for ip_row in html.find('#list table tr'):

            ip_element = ip_row.find('td[data-title="IP"]', first=True)
            port_element = ip_row.find('td[data-title="PORT"]', first=True)

            if ip_element and port_element:
                p = ProxyIP(ip=ip_element.text, port=port_element.text)
                ip_list.append(p)

        return ip_list

    @staticmethod
    def should_render_js() -> bool:
        return False



from requests_html import HTML

from ..database import ProxyIP


class BaseProvider(object):
    """BaseProvider is the abstract class for the proxy providers

    :raises NotImplementedError: [if urls() or parse() is not implemented]
    """

    _sleep = 0

    def __init__(self):
        pass

    def __str__(self):
        return self.__class__.__name__

    def sleep_seconds(self) -> int:
        """Return a sleep time for each request, by default it is 0

        :return: sleep time in seconds
        """
        return self._sleep

    def urls(self) -> [str]:
        """Return a list of url strings for crawling

        :return: [a list of url strings]
        :rtype: [str]
        """

        raise NotImplementedError

    def parse(self, html: HTML) -> [ProxyIP]:
        """Parse the document in order to get a list of proxies

        :param html: the HTML object from requests-html
        :return: a list of proxy ips
        """

        raise NotImplementedError

    @staticmethod
    def should_render_js() -> bool:
        """Whether needs js rendering
        By default, it is False.

        :return: a boolean value indicating whether or not js rendering is needed
        :rtype: bool
        """

        return False



from requests_html import HTML

from scylla.database import ProxyIP
from .base_provider import BaseProvider


class FreeProxyListProvider(BaseProvider):

    def parse(self, html: HTML) -> [ProxyIP]:
        ip_list: [ProxyIP] = []

        for ip_row in html.find('#proxylisttable tbody tr'):
            ip_address = ip_row.find('td:nth-child(1)', first=True).text
            port = ip_row.find('td:nth-child(2)', first=True).text

            p = ProxyIP(ip=ip_address, port=port)

            ip_list.append(p)

        return ip_list

    def urls(self) -> [str]:
        return [
            'https://free-proxy-list.net/'
        ]



import json
import math

import requests

from .loggings import logger
from .tcpping import ping

IP_CHECKER_API = 'http://ipinfo.io'
IP_CHECKER_API_SSL = 'https://ipinfo.io'

__CURRENT_IP__ = None


def get_current_ip():
    global __CURRENT_IP__
    if __CURRENT_IP__:
        logger.debug('get _current_ip')
        return __CURRENT_IP__
    else:
        logger.debug('fetch _current_ip')
        r = requests.get(IP_CHECKER_API)
        j = json.loads(r.text)
        __CURRENT_IP__ = j['ip']
        return __CURRENT_IP__


class Validator(object):
    def __init__(self, host: str, port: int, using_ssl: bool = False):
        self._host = host
        self._port = port

        self._checking_api = IP_CHECKER_API_SSL if using_ssl else IP_CHECKER_API

        # default values
        self._success_rate = 0.0
        self._latency = float('inf')

        self._anonymous = False
        self._valid = False

        self._meta = None

    def validate_latency(self):
        try:
            (self._latency, self._success_rate) = ping(self._host, self._port)
        except ConnectionRefusedError:
            self._latency, self._success_rate = math.inf, 0.0

    def validate_proxy(self):
        proxy_str = 'http://{}:{}'.format(self._host, self._port)
        try:
            r = requests.get(IP_CHECKER_API, proxies={'https': proxy_str, 'http': proxy_str}, verify=False, timeout=15)
            if r.ok:
                j = json.loads(r.text)

                if j['ip'] != get_current_ip():
                    self._anonymous = True
                self._valid = True

                # Load meta data
                # TODO: better location check
                meta = {
                    'location': j['loc'],
                    'organization': j['org'],
                    'region': j['region'],
                    'country': j['country'],
                    'city': j['city'],
                }
                self._meta = meta

        except requests.Timeout:
            logger.debug('Catch requests.Timeout for proxy ip: {}'.format(self._host))
        except requests.RequestException as e:
            logger.debug('Catch requests.RequestException for proxy ip: {}'.format(self._host))
            logger.debug(e.__str__())

    def validate(self):
        self.validate_latency()
        self.validate_proxy()

    @property
    def latency(self):
        return self._latency

    @property
    def success_rate(self):
        return self._success_rate

    @property
    def valid(self):
        return self._valid

    @property
    def anonymous(self):
        return self._anonymous

    @property
    def meta(self):
        return self._meta



from requests_html import HTML

from scylla.database import ProxyIP
from scylla.providers import BaseProvider


class Data5uProvider(BaseProvider):

    def urls(self) -> [str]:
        return [
            'http://www.data5u.com/free/gwgn/index.shtml',
            'http://www.data5u.com/free/gngn/index.shtml',
        ]

    def parse(self, html: HTML) -> [ProxyIP]:
        ip_list: [ProxyIP] = []

        for ip_row in html.find('.wlist > ul > li:nth-child(2) .l2'):

            ip_element = ip_row.find('span:nth-child(1)', first=True)
            port_element = ip_row.find('span:nth-child(2)', first=True)

            if ip_element and port_element:
                p = ProxyIP(ip=ip_element.text, port=port_element.text)
                ip_list.append(p)

        return ip_list

    @staticmethod
    def should_render_js() -> bool:
        return False



import socket
import time
from timeit import default_timer as timer

from six.moves import zip_longest


def avg(x):
    return sum(x) / float(len(x))


class Socket(object):
    def __init__(self, family, type_, timeout):
        s = socket.socket(family, type_)
        s.settimeout(timeout)
        self._s = s

    def connect(self, host, port=80):
        self._s.connect((host, int(port)))

    def shutdown(self):
        self._s.shutdown(socket.SHUT_RD)

    def close(self):
        self._s.close()


class Timer(object):
    def __init__(self):
        self._start = 0
        self._stop = 0

    def start(self):
        self._start = timer()

    def stop(self):
        self._stop = timer()

    def cost(self, funcs, args):
        # TODO: handle ConnectionRefusedError
        self.start()
        for func, arg in zip_longest(funcs, args):
            if arg:
                func(*arg)
            else:
                func()

        self.stop()
        return self._stop - self._start


class Ping(object):
    def __init__(self, host: str, port: int, timeout=1):
        self.timer = Timer()

        self._successes = 0
        self._failed = 0
        self._conn_times = []
        self._host = host
        self._port = port
        self._timeout = timeout

    def _create_socket(self, family, type_):
        return Socket(family, type_, self._timeout)

    def _success_rate(self):
        count = self._successes + self._failed
        try:
            rate = float(self._successes) / count
            rate = '{0:.2f}'.format(rate)
        except ZeroDivisionError:
            rate = '0.00'
        return float(rate)

    def _get_conn_times(self) -> [float]:
        return self._conn_times if self._conn_times != [] else [0]

    def get_maximum(self) -> float:
        return max(self._get_conn_times())

    def get_minimum(self) -> float:
        return min(self._get_conn_times())

    def get_average(self) -> float:
        return avg(self._get_conn_times())

    def get_success_rate(self):
        return self._success_rate()

    def ping(self, count=10, sleep=0.3):
        for n in range(1, count + 1):
            s = self._create_socket(socket.AF_INET, socket.SOCK_STREAM)
            try:
                time.sleep(sleep)
                cost_time = self.timer.cost(
                    (s.connect, s.shutdown),
                    ((self._host, self._port), None))
                s_runtime = 1000 * cost_time

                self._conn_times.append(s_runtime)
            except socket.timeout:
                self._failed += 1
            except ConnectionResetError:
                self._failed += 1
            else:
                self._successes += 1

            finally:
                s.close()


def ping(host: str, port: int, count: int = 10, sleep: float = 0.2) -> (int, float):
    """
    Ping a server and port with tcp socket
    :param host: the hostname
    :param port: the port number
    :param count: number of connection tries, by default it is 10
    :param sleep: length of sleep time in between sequent pings, by default it is 0.3
    :return: a tuple for (average_latency, success_rate)
    """
    p = Ping(host=host, port=port)
    p.ping(count=count, sleep=sleep)
    return p.get_average(), p.get_success_rate()



import time
from concurrent.futures import ThreadPoolExecutor
from multiprocessing import Queue, Process
from threading import Thread

import pyppeteer
import schedule

from scylla.database import ProxyIP
from scylla.jobs import validate_proxy_ip
from scylla.loggings import logger
from scylla.providers import *
from scylla.worker import Worker


def fetch_ips(q: Queue, validator_queue: Queue):
    logger.debug('fetch_ips...')
    worker = Worker()

    while True:
        try:
            provider: BaseProvider = q.get()

            provider_name = provider.__class__.__name__

            logger.debug('Get a provider from the provider queue: ' + provider_name)

            for url in provider.urls():

                html = worker.get_html(url)

                if html:
                    proxies = provider.parse(html)

                    for p in proxies:
                        validator_queue.put(p)
                        logger.debug('Put new proxy ip into queue: {}'.format(p.__str__()))

                    logger.info(
                        ' {}: feed {} potential proxies into the validator queue'.format(provider_name, len(proxies))
                    )
        except (KeyboardInterrupt, InterruptedError, SystemExit):
            logger.info('worker_process exited.')
            break
        except pyppeteer.errors.PyppeteerError as e:
            logger.debug('pyppeteer.errors.PyppeteerError detected: {}\n'
                         + 'Please make sure you have installed all the dependencies for chromium correctly'.format(e))
        except Exception as e:
            worker = Worker()  # reset worker
            logger.warning('Unhandled exception is detected: {}'.format(e))


def validate_ips(q: Queue, validator_pool: ThreadPoolExecutor):
    while True:
        try:
            proxy: ProxyIP = q.get()

            validator_pool.submit(validate_proxy_ip, p=proxy)
        except KeyboardInterrupt:
            break


def cron_schedule(scheduler, only_once=False):
    """

    :param scheduler: the Scheduler instance
    :param only_once: flag for testing
    """

    def feed():
        scheduler.feed_providers()

    # feed providers at the very beginning
    scheduler.feed_providers()

    schedule.every(10).minutes.do(feed)

    logger.info('Start python scheduler')

    flag = True

    while flag:
        try:
            schedule.run_pending()

            if only_once:
                flag = False
            else:
                time.sleep(60)
        except (KeyboardInterrupt, InterruptedError):
            logger.info('Stopping python scheduler')
            break


class Scheduler(object):

    def __init__(self):
        self.worker_queue = Queue()
        self.validator_queue = Queue()
        self.worker_process = None
        self.validator_thread = None
        self.cron_thread = None
        self.validator_pool = ThreadPoolExecutor(max_workers=20)

    def start(self):
        """
        Start the scheduler with processes for worker (fetching candidate proxies from different providers),
        and validator threads for checking whether the fetched proxies are able to use.

        """
        logger.info('Scheduler starts...')

        self.cron_thread = Thread(target=cron_schedule, args=(self,), daemon=True)
        self.worker_process = Process(target=fetch_ips, args=(self.worker_queue, self.validator_queue))
        self.validator_thread = Thread(target=validate_ips, args=(self.validator_queue, self.validator_pool))

        self.cron_thread.daemon = True
        self.worker_process.daemon = True
        self.validator_thread.daemon = True

        self.cron_thread.start()
        self.worker_process.start()  # Python will wait for all process finished
        logger.info('worker_process started')
        self.validator_thread.start()
        logger.info('validator_thread started')

    def join(self):
        """
        Wait for worker processes and validator threads

        """
        while (self.worker_process and self.worker_process.is_alive()) or (
                self.validator_thread and self.validator_thread.is_alive()):
            self.worker_process.join()
            self.validator_thread.join()

    def feed_providers(self):
        logger.debug('feed {} providers...'.format(len(all_providers)))

        for provider in all_providers:
            self.worker_queue.put(provider())

    def stop(self):
        self.worker_queue.close()
        self.worker_process.terminate()
        # self.validator_thread.terminate() # TODO: 'terminate' the thread using a flag
        self.validator_pool.shutdown(wait=False)



import argparse
import sys

from scylla.config import batch_set_config
from scylla.database import create_db_tables
from scylla.loggings import logger
from scylla.scheduler import Scheduler
from scylla.web import start_web_server
from ._version import __version__

CMD_DESCRIPTION = """Scylla command line mode
This command could start a scheduler for crawling and validating proxies.
In addition, a web server with APIs can also be launched.

"""


def main(args) -> int:
    parser = argparse.ArgumentParser(description=CMD_DESCRIPTION,
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('--no-webserver', '-no-ws', action='store_true',
                        help='Prevent starting a web server for JSON API')
    parser.add_argument('--web-port', '-wp', type=int, default=8000,
                        help='The port number for the web server')
    parser.add_argument('--web-host', '-wh', type=str, default='0.0.0.0',
                        help='The hostname for the web server')
    parser.add_argument('--version', '-v', action='store_true',
                        help='Print the version of Scylla')

    parsed_args = parser.parse_args(args)

    parsed_args_dict = vars(parsed_args)

    batch_set_config(**vars(parsed_args))

    handle_special_flags(parsed_args_dict)

    create_db_tables()

    s = Scheduler()

    try:
        s.start()

        # web server
        if not parsed_args_dict['no_webserver']:
            logger.info('Start the web server')
            start_web_server(
                host=parsed_args_dict['web_host'], port=parsed_args_dict['web_port'])

        s.join()
    except (KeyboardInterrupt, SystemExit):
        logger.info('catch KeyboardInterrupt, exiting...')
        s.stop()
        return 0

    return 0


def handle_special_flags(args: dict):
    if args['version']:
        print('v{}'.format(__version__))
        sys.exit(0)


def app_main():
    sys.exit(main(sys.argv[1:]))



import re

from requests_html import HTML

from scylla.database import ProxyIP
from scylla.providers import BaseProvider


class SpysOneProvider(BaseProvider):

    def urls(self) -> [str]:
        return [
            'http://spys.one/en/anonymous-proxy-list/',
            # 'http://spys.one/en/http-proxy-list/',
            # 'http://spys.one/en/https-ssl-proxy/',
        ]

    def parse(self, html: HTML) -> [ProxyIP]:
        ip_list: [ProxyIP] = []

        print('html:', html.raw_html)

        for ip_row in html.find('table tr[onmouseover]'):

            ip_port_text_elem = ip_row.find('.spy14', first=True)

            if ip_port_text_elem:
                ip_port_text = ip_port_text_elem.text

                print('ip_port_text: "', ip_port_text, '"')

                ip = re.search(r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}', ip_port_text).group(0)
                port = re.search(r':\n(\d{2,5})', ip_port_text).group(1)

                if ip and port:
                    p = ProxyIP(ip=ip, port=port)
                    ip_list.append(p)

        return ip_list

    @staticmethod
    def should_render_js() -> bool:
        return True



import io
import os

from setuptools import setup

import scylla

here = os.path.abspath(os.path.dirname(__file__))
# Import the README and use it as the long-description.
with io.open(os.path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = '\n' + f.read()

with open('requirements.txt') as f:
    required = f.read().splitlines()

setup(
    name='scylla',
    python_requires='>=3.6.0',
    # If your package is a single module, use this instead of 'packages':
    packages=[
        'scylla',
        'scylla.providers',
        'scylla.web'
    ],
    entry_points={
        'console_scripts': ['scylla = scylla.cli:app_main']
    },
    version=scylla.__version__,
    description='Intelligent proxy pool for Humans™',
    long_description=long_description,
    author=scylla.__author__,
    author_email='wildcat.name@gmail.com',
    url='https://github.com/imWildCat/scylla',
    # download_url='https://github.com/imWildCat/scylla/archive/0.1.0.tar.gz',
    keywords=['proxy', 'api', 'scylla'],
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'License :: OSI Approved :: Apache Software License'
    ],
    install_requires=required,
    include_package_data=True,
)



import os

from playhouse.shortcuts import model_to_dict
from sanic import Sanic
from sanic.request import Request
from sanic.response import json
from sanic_cors import CORS

from scylla.database import ProxyIP
from scylla.loggings import logger

app = Sanic()

CORS(app)

base_path = os.path.abspath(os.path.join(__file__, os.pardir, os.pardir))

app.static('/', base_path + '/assets/index.html')
app.static('/*', base_path + '/assets')


# app.url_for('', filename='index.html')


# @app.route('/')
# async def test(request: Request):
#     return json({'hello': 'world', 'args': request.raw_args})

def _parse_str_to_int(s: str) -> int:
    try:
        return int(s)
    except ValueError:
        return 0


@app.route('/api/v1/proxies')
async def api_v1_proxies(request: Request):
    args = request.raw_args

    limit = 20

    page = 1

    is_anonymous = 2  # 0: no, 1: yes, 2: any

    if 'limit' in args:
        int_limit = _parse_str_to_int(args['limit'])
        limit = int_limit if int_limit else 20

    if 'page' in args:
        int_page = _parse_str_to_int(args['page'])
        page = int_page if int_page > 0 else 1

    if 'anonymous' in args:
        str_anonymous = args['anonymous']
        if str_anonymous == 'true':
            is_anonymous = 1
        elif str_anonymous == 'false':
            is_anonymous = 0
        else:
            is_anonymous = 2

    proxies = ProxyIP.select().where(ProxyIP.latency > 0).where(ProxyIP.latency < 9999) \
        .where(ProxyIP.is_valid == True)

    if is_anonymous != 2:
        if is_anonymous == 1:
            proxies = proxies.where(ProxyIP.is_anonymous == True)
        elif is_anonymous == 0:
            proxies = proxies.where(ProxyIP.is_anonymous == False)

    proxies = proxies.order_by(ProxyIP.updated_at.desc(), ProxyIP.latency).offset(page - 1).limit(limit)

    logger.debug('Perform SQL query: {}'.format(proxies.sql()))

    proxy_list = []

    for p in proxies:
        proxy_list.append(model_to_dict(p))

    return json({
        'proxies': proxy_list,
    })


def start_web_server(host='0.0.0.0', port=8000):
    app.run(host=host, port=port)



import re

from requests_html import HTML

from scylla.database import ProxyIP
from .base_provider import BaseProvider


class CoolProxyProvider(BaseProvider):

    def parse(self, html: HTML) -> [ProxyIP]:
        ip_list: [ProxyIP] = []

        for ip_row in html.find('table tr'):

            ip_element = ip_row.find('td:nth-child(1)', first=True)
            port_element = ip_row.find('td:nth-child(2)', first=True)

            if ip_element and port_element:
                p = ProxyIP(ip=re.sub(r'document\.write\(.+\)', '', ip_element.text), port=port_element.text)

                ip_list.append(p)

        return ip_list

    def urls(self) -> [str]:
        return [
            'https://www.cool-proxy.net/proxies/http_proxy_list/country_code:/port:/anonymous:1',
            'https://www.cool-proxy.net/proxies/http_proxy_list/country_code:/port:/anonymous:1/page:2',
            'https://www.cool-proxy.net/proxies/http_proxy_list/country_code:/port:/anonymous:1/page:3',
        ]

    @staticmethod
    def should_render_js() -> bool:
        return True
